@@ -170,8 +170,9 @@
    display-trace ; for backwards compatibility describe-request describe-event describe-reply
    closed-display-p
    ;; extensions
-   open-default-display with-open-display
+   open-default-display with-open-display events-queued
    display-get-default display-resource-manager-string screen-resource-string
+   *canonicalize-encoding*
    ;;; Only when using libXt:
    ;; last-event-processed last-timestamp-processed
    ;;; Only when using the native resource database, not resource.lisp:
@@ -604,23 +605,29 @@
   `(set-string-property ,window :WM_CLIENT_MACHINE ,name))
 
 (defun get-wm-class (window)
-  (let ((value (get-property window :WM_CLASS :type :STRING :result-type 'string :transform #'card8->char)))
+  (let ((value (get-property window :WM_CLASS :type :STRING
+                             :result-type 'string :transform #'card8->char)))
     (when value
-      (let* ((name-len (position (load-time-value (card8->char 0)) (the string value)))
-             (name (subseq (the string value) 0 name-len))
-             (class (subseq (the string value) (1+ name-len) (1- (length value)))))
+      ;; Buggy clients may not comply with the format,
+      ;; so deal with the unexpected.
+      (let* ((first-zero (position #\Null (the string value)))
+             (second-zero (and first-zero
+                               (position #\Null (the string value)
+                                         :start (1+ first-zero))))
+             (name (subseq (the string value) 0 first-zero))
+             (class (and first-zero
+                         (subseq (the string value) (1+ first-zero)
+                                 second-zero))))
         (values (and (plusp (length name)) name)
                 (and (plusp (length class)) class))))))
 
 (defun set-wm-class (window resource-name resource-class)
   (set-string-property window :WM_CLASS
                        (ext:string-concat
-                        (string (or resource-name ""))
-                        (load-time-value
-                         (make-string 1 :initial-element (card8->char 0)))
-                        (string (or resource-class ""))
-                        (load-time-value
-                         (make-string 1 :initial-element (card8->char 0)))))
+                        (string (or resource-name #1=""))
+                        #2=#,(string #\Null)
+                        (string (or resource-class #1#))
+                        #2#))
   (values))
 
 (defun wm-command (window)
@@ -633,7 +640,7 @@
         (end 0)
         (len (length command-string)))
        ((>= start len) (nreverse command))
-    (setq end (position (load-time-value (card8->char 0)) command-string :start start))
+    (setq end (position #\Null command-string :start start))
     (push (subseq command-string start end) command)))
 
 (defsetf wm-command set-wm-command)
@@ -649,7 +656,7 @@
       (with-standard-io-syntax
         (dolist (c command)
           (prin1 c stream)
-          (write-char (load-time-value (card8->char 0)) stream)))))
+          (write-char #\Null stream)))))
   command)
 
 ;;-----------------------------------------------------------------------------
@@ -1605,13 +1612,6 @@ default display as given by GET-DEFAULT-DISPLAY otherwise."
 	      (or (x-type-error-type-string condition)
 		  (type-error-expected-type condition))))))
 
-(define-condition closed-display (x-error)
-  ((display :reader closed-display-display :initarg :display))
-  (:report
-    (lambda (condition stream)
-      (format stream "Attempt to use closed display ~s"
-	      (closed-display-display condition)))))
-
 (define-condition lookup-error (x-error)
   ((id :reader lookup-error-id :initarg :id)
    (display :reader lookup-error-display :initarg :display)
@@ -1747,10 +1747,6 @@ default display as given by GET-DEFAULT-DISPLAY otherwise."
 
 (define-condition implementation-error (request-error) ())
 
-(define-condition connection-failure (x-error) ())
-
-(define-condition device-busy (x-error) ())
-
 ;; (define-condition server-disconnect (x-error) ())
 
 (pushnew "XLIB" custom:*system-package-list* :test #'string=)
@@ -1766,5 +1762,17 @@ default display as given by GET-DEFAULT-DISPLAY otherwise."
 (undefined DRAW-GLYPH)
 (undefined DRAW-IMAGE-GLYPH)
 (undefined TRANSLATE-DEFAULT)
-(undefined QUEUE-EVENT)
 )
+
+;; canonicalize encodings supplied by X, see clx.f:get_font_info_and_display()
+(defvar *canonicalize-encoding*
+  ;; this encoding canonicalization was requested by
+  ;; Pascal J.Bourguignon <pjb@informatimago.com> in
+  ;; http://article.gmane.org/gmane.lisp.clisp.general:7794
+  ;; https://sourceforge.net/p/clisp/mailman/message/11011372/
+  `(,(lambda (s)
+       (if (and (<= 4 (length s))
+                (string-equal s "iso" :end1 3)
+                (not (char= #\- (char s 3))))
+           (concatenate 'string "ISO-" (subseq s 3))
+           s))))
