@@ -2,19 +2,15 @@
  * GDBM - The GNU database manager
  * <http://www.gnu.org/software/gdbm/>
  * Copyright (C) 2007  Masayuki Onjo <onjo@lispuser.net>
- * Copyright (C) 2007-2008  Sam Steingold <sds@gnu.org>
+ * Copyright (C) 2007-2008, 2010-2012  Sam Steingold <sds@gnu.org>
  * GPL2
  */
 
 #include "clisp.h"
 #include "config.h"
 
-#if defined(HAVE_STRING_H)
-# include <string.h>            /* memcpy */
-#endif
-#if defined(HAVE_UNISTD_H)
-# include <unistd.h>            /* for lseek */
-#endif
+#include <string.h>             /* memcpy */
+#include <unistd.h>             /* for lseek */
 
 #if defined(HAVE_GDBM_H)
 # include <gdbm.h>
@@ -33,7 +29,7 @@ typedef enum {
    * retrieved when #*101 was stored), and the length would take a full
    * word which would nix any potential saving from using bits instead
    * of full bytes (this is similar to sub-byte :ELEMENT-TYPE for streams,
-   * see <http://clisp.cons.org/impnotes/stream-dict.html#eltype>).
+   * see <http://clisp.org/impnotes/stream-dict.html#eltype>).
    * Also, Atype_16Bit is not exported from lispbibl.
    *     GDBM_DATA_BIT_VECTOR,
    *     GDBM_DATA_2BIT_VECTOR,
@@ -70,7 +66,7 @@ DEFCHECKER(check_gdbm_errno, prefix=GDBM, NO-ERROR MALLOC-ERROR              \
            READER-CANT-REORGANIZE UNKNOWN-UPDATE ITEM-NOT-FOUND              \
            REORGANIZE-FAILED CANNOT-REPLACE ILLEGAL-DATA OPT-ALREADY-SET     \
            OPT-ILLEGAL)
-nonreturning_function(static, error_gdbm, (char *fatal_message)) {
+static _Noreturn void error_gdbm (char *fatal_message) {
   end_blocking_system_call(); /* in case we are called from _gdbm_fatal() */
   pushSTACK(`GDBM::GDBM-ERROR`);
   pushSTACK(`:MESSAGE`);
@@ -88,8 +84,10 @@ nonreturning_function(static, error_gdbm, (char *fatal_message)) {
   NOTREACHED;
 }
 
+DEFVAR(gdbm_version_cache,NIL)
 DEFUN(GDBM::GDBM-VERSION,)
-{ VALUES1(safe_to_string(gdbm_version)); }
+{ VALUES1(!nullp(O(gdbm_version_cache)) ? O(gdbm_version_cache)
+          : (O(gdbm_version_cache) = safe_to_string(gdbm_version))); }
 
 /* can trigger GC */
 #define GDBM_SLOT_FILE  1
@@ -216,7 +214,7 @@ static object coerce_bitvector (object arg) {
   }
 }
 
-nonreturning_function(static, error_bad_type, (object lisp_obj)) {
+static _Noreturn void error_bad_type (object lisp_obj) {
   pushSTACK(`GDBM::GDBM-ERROR`);
   pushSTACK(`:MESSAGE`);
   pushSTACK(`"invalid lisp object type: "`);
@@ -296,8 +294,7 @@ static object datum_to_object (datum d, gdbm_data_t data_type) {
       return o;
     }
     case GDBM_DATA_VECTOR: case GDBM_DATA_8BIT_VECTOR: {
-      object o = allocate_bit_vector(Atype_8Bit,d.dsize);
-      SYSCALL(memcpy(TheSbvector(o)->data,d.dptr,d.dsize));
+      object o = data_to_sb8vector(d.dptr,d.dsize);
       free(d.dptr);
       return o;
     }
@@ -313,8 +310,7 @@ static object datum_to_object (datum d, gdbm_data_t data_type) {
         funcall(L(error_of_type), 8);
         NOTREACHED;
       } else {
-        object o = allocate_bit_vector(Atype_32Bit,d.dsize/4);
-        SYSCALL(memcpy(TheSbvector(o)->data,d.dptr,d.dsize));
+        object o = data_to_sbvector(Atype_32Bit,d.dsize/4,d.dptr,d.dsize);
         free(d.dptr);
         return o;
       }
