@@ -1,23 +1,16 @@
 /*
  * ======= General internationalization, for Lisp programs too =======
  * Copyright (C) 1990-2005 Bruno Haible
- * Copyright (C) 1998-2008 Sam Steingold
+ * Copyright (C) 1998-2008, 2010, 2012 Sam Steingold
  * GPL2
  */
 
-#if defined(_WIN32)
-/* get ASCII functions */
-# undef UNICODE
-#endif
-
 #include "clisp.h"
 #include "config.h"
 
 #include <string.h>             /* strncpy() */
 #include <locale.h>
-#if defined(HAVE_LANGINFO_H)
-# include <langinfo.h>
-#endif
+#include <langinfo.h>
 #include <limits.h>            /* for CHAR_MAX */
 
 #ifdef CLISP_UNICODE
@@ -42,13 +35,13 @@ static inline object do_gettext (const char* msgid,
   if (msgid[0] == '\0') {
     translated_msg = "";  /* Don't return the catalog's header entry. */
   } else {
-    begin_system_call();
+    begin_blocking_system_call();
 #  ifdef CLISP_UNICODE
     if (domain != NULL)
       bind_textdomain_codeset(domain,"UTF-8");
 #  endif
     translated_msg = dcgettext(domain,msgid,category);
-    end_system_call();
+    end_blocking_system_call();
   }
   return asciz_to_string(translated_msg,Symbol_value(S(utf_8)));
 }
@@ -57,13 +50,13 @@ static inline object do_ngettext (const char* msgid, const char* msgid_plural,
                                   const char* domain, uint32 n, int category)
 {
   const char* translated_msg;
-  begin_system_call();
+  begin_blocking_system_call();
 # ifdef CLISP_UNICODE
   if (domain != NULL)
     bind_textdomain_codeset(domain,"UTF-8");
 # endif
   translated_msg = dcngettext(domain,msgid,msgid_plural,n,category);
-  end_system_call();
+  end_blocking_system_call();
   return asciz_to_string(translated_msg,Symbol_value(S(utf_8)));
 }
 
@@ -201,7 +194,6 @@ DEFUN(I18N:SET-TEXTDOMAINDIR, domain directory)
 
 
 /* ======================== locale ======================== */
-
 DEFUN(I18N:SET-LOCALE, &optional category locale)
 { /* call setlocale(3) */
   gcv_object_t *category = &STACK_1;
@@ -215,7 +207,7 @@ DEFUN(I18N:SET-LOCALE, &optional category locale)
         res = setlocale(check_locale_category_map.table[pos].c_const,NULL);
         end_system_call();
         pushSTACK(*check_locale_category_map.table[pos].l_const);
-        pushSTACK(res ? asciz_to_string(res,GLO(misc_encoding)) : NIL);
+        pushSTACK(safe_to_string(res));
       }
     } else {
       *locale = check_string(*locale);
@@ -225,7 +217,7 @@ DEFUN(I18N:SET-LOCALE, &optional category locale)
             res = setlocale(check_locale_category_map.table[pos].c_const,loc_z);
             end_system_call();
             pushSTACK(*check_locale_category_map.table[pos].l_const);
-            pushSTACK(res ? asciz_to_string(res,GLO(misc_encoding)) : NIL);
+            pushSTACK(safe_to_string(res));
           }
         });
     }
@@ -244,7 +236,7 @@ DEFUN(I18N:SET-LOCALE, &optional category locale)
           end_system_call();
         });
     }
-    VALUES1(res ? asciz_to_string(res,GLO(misc_encoding)) : NIL);
+    VALUES1(safe_to_string(res));
   }
   skipSTACK(2);
 }
@@ -254,12 +246,14 @@ DEFUN(I18N:SET-LOCALE, &optional category locale)
  res is a malloc'ed area of size res_size
  it may be increased as a result of calling this function */
 # define GET_LOCALE_INFO_BUF_SIZE 256
-static void get_locale_info (int what, char**res, int *res_size) {
+static int get_locale_info (int what, char**res, int *res_size, int errorp) {
   int val;
   begin_system_call();
   val = GetLocaleInfo(LOCALE_SYSTEM_DEFAULT,what,*res,*res_size);
   end_system_call();
-  if (val == 0) OS_error();
+  if (val == 0)
+    if (errorp) OS_error();
+    else return 1;
   if (val > *res_size) {
     *res = (char*)clisp_realloc(*res,val);
     if (*res == NULL) OS_error();
@@ -268,6 +262,7 @@ static void get_locale_info (int what, char**res, int *res_size) {
     GetLocaleInfo(LOCALE_SYSTEM_DEFAULT,what,*res,*res_size);
     end_system_call();
   }
+  return 0;
 }
 #endif
 
@@ -281,20 +276,20 @@ static /*maygc*/ object bool_char_lconv(char val) {
   switch (val) {
     case 0: return NIL;
     case 1: return T;
-    case CHAR_MAX: return S(Kunspecific);
+    case CHAR_MAX: case -1: return S(Kunspecific);
     default:
       pushSTACK(CLSTEXT("~S: localeconv() returned an invalid value ~S (should be one of ~S, ~S, CHAR_MAX=~S)"));
       pushSTACK(TheSubr(subr_self)->name);
-      pushSTACK(fixnum(val));
+      pushSTACK(sfixnum(val));
       pushSTACK(Fixnum_0); pushSTACK(Fixnum_1); pushSTACK(fixnum(CHAR_MAX));
       funcall(S(warn),6);
-      return fixnum(val);
+      return sfixnum(val);
   }
 }
 static object int_char_lconv(char val) {
   switch (val) {
-    case CHAR_MAX: return S(Kunspecific);
-    default: return fixnum(val);
+    case CHAR_MAX: case -1: return S(Kunspecific);
+    default: return sfixnum(val);
   }
 }
 DEFUN(I18N:LOCALE-CONV,)
@@ -361,26 +356,26 @@ static void thousands_sep_to_STACK (int what, char** gres, int* res_size) {
   /* "1;2;3" ==> #(1 2 3) */
   int start = 0, end = 0, count = 0, limit;
   char *res;
-  get_locale_info(what,gres,res_size);
+  get_locale_info(what,gres,res_size,1);
   res = *gres; limit = *res_size;
   while (res[end] && (end < limit)) {
     while (res[end] && (res[end] != ';') && (end < limit)) end++;
-    pushSTACK(fixnum(my_atoi(res+start))); count++;
+    pushSTACK(sfixnum(my_atoi(res+start))); count++;
     if (!res[end]) break;
     start = ++end;
   }
   value1 = vectorof(count); pushSTACK(value1);
 }
 static void locale_string_to_STACK (int what, char**res, int* res_size) {
-  get_locale_info(what,res,res_size);
+  get_locale_info(what,res,res_size,1);
   pushSTACK(asciz_to_string(*res,GLO(misc_encoding)));
 }
 static void locale_int_to_STACK (int what, char**res, int* res_size) {
-  get_locale_info(what,res,res_size);
-  pushSTACK(fixnum(my_atoi(*res)));
+  get_locale_info(what,res,res_size,1);
+  pushSTACK(sfixnum(my_atoi(*res)));
 }
 static void locale_bool_to_STACK (int what, char**res, int* res_size) {
-  get_locale_info(what,res,res_size);
+  get_locale_info(what,res,res_size,1);
   pushSTACK(my_atoi(*res) ? T : NIL);
 }
 DEFUN(I18N:LOCALE-CONV,)
@@ -416,7 +411,6 @@ DEFUN(I18N:LOCALE-CONV,)
 }
 #endif
 
-#if defined(HAVE_NL_LANGINFO) || defined(WIN32_NATIVE)
 DEFCHECKER(check_nl_item,CODESET                                        \
            D_T_FMT D_FMT T_FMT T_FMT_AMPM AM_STR PM_STR                 \
            DAY_1 DAY_2 DAY_3 DAY_4 DAY_5 DAY_6 DAY_7                    \
@@ -472,17 +466,18 @@ DEFCHECKER(check_nl_item,CODESET                                        \
            LOCALE_SNEGATIVESIGN LOCALE_SPOSITIVESIGN                    \
            LOCALE_SSHORTDATE LOCALE_SSORTNAME LOCALE_STHOUSAND          \
            LOCALE_STIME LOCALE_STIMEFORMAT LOCALE_SYEARMONTH)
-#if defined(HAVE_NL_LANGINFO)
-# define get_lang_info(what)                                            \
-  begin_system_call(); res = nl_langinfo(what); end_system_call()
-# define res_to_obj() (res ? asciz_to_string(res,GLO(misc_encoding)) : NIL)
-# define DECLARE_RES  char* res
-# define FINISH_RES
-#elif defined(WIN32_NATIVE)
-# define get_lang_info(what)  get_locale_info(what,&res,&res_size)
-# define res_to_obj() (asciz_to_string(res,GLO(misc_encoding)))
+#if defined(WIN32_NATIVE)
+# define get_lang_info(what,errorp)                                     \
+  if (get_locale_info(what,&res,&res_size,errorp)) funcall(`OS::ERRNO`,0); \
+  else VALUES1(asciz_to_string(res,GLO(misc_encoding)))
 # define DECLARE_RES  int res_size=GET_LOCALE_INFO_BUF_SIZE; char *res=(char*)clisp_malloc(res_size)
 # define FINISH_RES   begin_system_call(); free(res); end_system_call()
+#else
+# define get_lang_info(what,errorp)                                     \
+  begin_system_call(); res = nl_langinfo(what); end_system_call();      \
+  VALUES1(safe_to_string(res))
+# define DECLARE_RES  char* res
+# define FINISH_RES
 #endif
 DEFUNR(I18N:LANGUAGE-INFORMATION,&optional item)
 { /* call nl_langinfo(3) or GetLocaleInfo() */
@@ -491,18 +486,16 @@ DEFUNR(I18N:LANGUAGE-INFORMATION,&optional item)
     int pos = 0;
     DECLARE_RES;
     for (; pos < check_nl_item_map.size; pos++) {
-      get_lang_info(check_nl_item_map.table[pos].c_const);
       pushSTACK(*check_nl_item_map.table[pos].l_const);
-      pushSTACK(res_to_obj());
+      get_lang_info(check_nl_item_map.table[pos].c_const,0);
+      pushSTACK(value1);
     }
     FINISH_RES;
     VALUES1(listof(2*check_nl_item_map.size));
   } else {
     int item = check_nl_item(what);
     DECLARE_RES;
-    get_lang_info(item);
-    VALUES1(res_to_obj());
+    get_lang_info(item,1);
     FINISH_RES;
   }
 }
-#endif
