@@ -1,7 +1,7 @@
 ;;; LIBSVM interface
 ;;; <http://www.csie.ntu.edu.tw/~cjlin/libsvm/>
 ;;;
-;;; Copyright (C) 2006-2009 Sam Steingold <sds@gnu.org>
+;;; Copyright (C) 2006-2011 Sam Steingold <sds@gnu.org>
 ;;; This is Free Software, covered by the GNU GPL (v2)
 ;;; See http://www.gnu.org/copyleft/gpl.html
 
@@ -14,20 +14,29 @@
 (setf (documentation (find-package "LIBSVM") 'sys::impnotes) "libsvm")
 
 (default-foreign-language :stdc)
-(default-foreign-library
-    (namestring (make-pathname :name "svm" :defaults *load-pathname*
-                               :type #+win32 "dll" #+unix "so"
-                                     #-(or win32 unix) nil))) ; ???
 
-(c-lines "#include \"svm.h\"~%")
+(c-lines "#include \"config.h\"~%#include <svm.h>~%")
 
 (defvar *libsvm-output* *standard-output* "The stream for svm.so messages.")
 (cl:defun write-string-to-libsvm-output (s)
   (write-string s *libsvm-output*) (force-output *libsvm-output*))
 (def-call-in write-string-to-libsvm-output (:name "libsvm_print_string")
   (:arguments (s c-string)) (:return-type nil))
-(c-lines :init-always
-         "  svm_print_string = (void (*) (const char*))&libsvm_print_string;")
+(c-lines :init-always "
+ #if defined(HAVE_SVM_SET_PRINT_STRING_FUNCTION)
+  svm_set_print_string_function((void (*) (const char*))&libsvm_print_string);
+ #else
+  svm_print_string = (void (*) (const char*))&libsvm_print_string;
+ #endif~%")
+(c-lines "
+#if !defined(HAVE_SVM_DESTROY_MODEL)
+# if defined(HAVE_SVM_FREE_AND_DESTROY_MODEL)
+void svm_destroy_model (struct svm_model *model){ svm_free_and_destroy_model(&model); }
+# else
+#  error No svm_destroy_model and no svm_free_and_destroy_model!
+# endif
+#endif
+")
 
 ;;;
 ;;; types and constants
@@ -279,13 +288,13 @@ You must call (destroy-problem ret) yourself!
                  (parse-integer line :start pos :end colon))
                (multiple-value-setq (value pos)
                  (read-from-string line t nil :start (1+ colon)))
-               (setq maxindex (max maxindex index))
+               (when (< maxindex index) (setq maxindex index))
                (push (list (coerce index 'integer)
                            (coerce value 'double-float))
                      ret))
              (coerce (nreverse (cons (list -1 0d0) ret)) 'vector))
            x)))
-      (when log (format log "~:D record~:P~%" len)))
+      (when log (format log "~:D record~:P; MAX index=~:D~%" len maxindex)))
     (values (make-problem :l len :y (nreverse y) :x (nreverse x))
             maxindex)))
 
@@ -294,13 +303,17 @@ You must call (destroy-problem ret) yourself!
   target index1:value1 index2:value2 ..."
   (with-open-file (out file :direction :output)
     (let* ((size (problem-l problem))
+           (*read-default-float-format* 'double-float)
            (y (problem-y problem size)) (x (problem-x problem size)))
       (sys::start-message log 'save-problem file size "records")
       (dotimes (i size)
-        (format out "~G" (aref y i))
+        (write (aref y i) :stream out)
         (let ((nodes (aref x i)))
           (loop :for (index value) :across nodes :while (/= index -1) :do
-            (format out " ~D:~G" index value)))
+            (write-char #\Space out)
+            (write index :stream out)
+            (write-char #\: out)
+            (write value :stream out)))
         (terpri out)))
     (when log (format log "~:D byte~:P~%" (file-length out)))))
 
