@@ -1,7 +1,7 @@
 /*
  * PCRE - Perl Compatible Regular Expressions
  * <http://www.pcre.org/> up to v 8.01
- * Copyright (C) 2003-2010 Sam Steingold
+ * Copyright (C) 2003-2012 Sam Steingold
  * GPL2
  */
 
@@ -42,12 +42,14 @@ DEFUN(PCRE::PCRE-CONFIG, &optional what)
   if (missingp(STACK_0)) {
     int pos = 0;
     for (; pos < pcre_config_option_map.size; pos++) {
-      int ret;
-      begin_system_call();
-      pcre_config(pcre_config_option_map.table[pos].c_const,&ret);
-      end_system_call();
+      long ret;
+      int status;
       pushSTACK(*pcre_config_option_map.table[pos].l_const);
-      pushSTACK(L_to_I(ret));
+      begin_system_call();
+      status = pcre_config(pcre_config_option_map.table[pos].c_const,&ret);
+      end_system_call();
+      if (status == 0) pushSTACK(L_to_I(ret));
+      else pushSTACK(`:BADOPTION`);
     }
     VALUES1(listof(2*pcre_config_option_map.size));
   } else {
@@ -161,7 +163,7 @@ DEFCHECKER(error_pcre_code,prefix=PCRE_ERROR, NOMATCH NULL BADOPTION    \
            BADUTF8 BADUTF8-OFFSET PARTIAL BADPARTIAL :INTERNAL BADCOUNT \
            DFA-UITEM DFA-UCOND DFA-UMLIMIT DFA-WSSIZE DFA-RECURSE       \
            RECURSIONLIMIT NULLWSLIMIT BADNEWLINE)
-nonreturning_function(static, error_pcre, (int status)) {
+static _Noreturn void error_pcre (int status) {
   pushSTACK(error_pcre_code_reverse(status));
   pushSTACK(sfixnum(status)); pushSTACK(TheSubr(subr_self)->name);
   error(error_condition,"~S/~S=~S: ~S ~S");
@@ -218,17 +220,12 @@ static object fullinfo_firstbyte (pcre *c_pat, pcre_extra *study) {
 #endif
 static object fullinfo_firsttable (pcre *c_pat, pcre_extra *study) {
   unsigned char table[256];
-  object ret = allocate_bit_vector(Atype_Bit,256);
   int status;
-  void *data = TheSbvector(ret)->data;
-  /* no need to pin ret because pcre_fullinfo does not block */
-  handle_fault_range(PROT_READ_WRITE,(aint)data,(aint)data + sizeof(table));
   begin_system_call();
   status = pcre_fullinfo(c_pat,study,PCRE_INFO_FIRSTTABLE,&table);
-  if (status >= 0) memcpy(data,table,sizeof(table));
   end_system_call();
   if (status < 0) error_pcre(status);
-  return ret;
+  return data_to_sb8vector(table,256);
 }
 static object fullinfo_lastliteral (pcre *c_pat, pcre_extra *study) {
   int value, status;
@@ -403,20 +400,21 @@ DEFUN(PCRE:PCRE-EXEC,pattern subject &key WORK-SPACE DFA BOOLEAN OFFSET \
   pcre_extra *study;
   skipSTACK(3); /* drop all options */
   check_pattern(&STACK_1,&c_pat,&study);
+  STACK_0 = check_string(STACK_0)
   begin_system_call();
   ret = pcre_fullinfo(c_pat,study,PCRE_INFO_CAPTURECOUNT,&capture_count);
   end_system_call();
   if (ret < 0) error_pcre(ret);
   /* when DFA is enabled, the number of matches is not easily estimated */
   ovector_size = 3 * (capture_count + (dfa_p ? workspace_size : 1));
- pcre_exec_retry:
-  ovector = (int*)alloca(sizeof(int)*ovector_size);
 # if !defined(HAVE_PCRE_DFA_EXEC)
   /* use pcre_exec instead */
 #  define pcre_dfa_exec(re,st,su,sb,of,op,ov,os,ws,wz) \
               pcre_exec(re,st,su,sb,of,op,ov,os)
 # endif
-  with_string_0(check_string(STACK_0),Symbol_value(S(utf_8)),subject, {
+  with_string_0(STACK_0,Symbol_value(S(utf_8)),subject, {
+     pcre_exec_retry:
+      ovector = (int*)alloca(sizeof(int)*ovector_size);
       begin_system_call();
       /* subject_bytelen is the length of subject in bytes,
          defined in with_string_0 */
@@ -427,27 +425,28 @@ DEFUN(PCRE:PCRE-EXEC,pattern subject &key WORK-SPACE DFA BOOLEAN OFFSET \
         : pcre_exec(c_pat,study,subject,subject_bytelen,offset,options,
                     ovector,ovector_size);
       end_system_call();
-    });
-  if (ret == PCRE_ERROR_NOMATCH) VALUES1(NIL);
-  else if (ret == 0) {          /* not enough space in ovector */
-    ovector_size <<= 1;
-    goto pcre_exec_retry;
-  } else if (ret > 0) {
-    if (bool_p) VALUES1(T); /* success indicator */
-    else { /* return a vector */
-      int pos, ov_pos;
-      ASSERT(ret <= ovector_size);
-      pushSTACK(allocate_vector(ret)); /* return value */
-      for (pos = ov_pos = 0; pos < ret; pos++, ov_pos+=2)
-        if (ovector[ov_pos] >= 0) {
-          pushSTACK(L_to_I(ovector[ov_pos]));
-          pushSTACK(L_to_I(ovector[ov_pos+1]));
-          funcall(`PCRE::MAKE-MATCH-BOA`,2);
-          TheSvector(STACK_0)->data[pos] = value1;
+      if (ret == PCRE_ERROR_NOMATCH) VALUES1(NIL);
+      else if (ret == 0) {          /* not enough space in ovector */
+        ovector_size <<= 1;
+        goto pcre_exec_retry;
+      } else if (ret > 0) {
+        if (bool_p) VALUES1(T); /* success indicator */
+        else { /* return a vector */
+          int pos; int ov_pos;  /* pacify the CPP parsing of macro with_string_0 */
+          ASSERT(ret <= ovector_size);
+          pushSTACK(allocate_vector(ret)); /* return value */
+          for (pos = ov_pos = 0; pos < ret; pos++, ov_pos+=2)
+            if (ovector[ov_pos] >= 0) {
+              const uintB* sub = (const uintB*)subject;
+              pushSTACK(L_to_I(Encoding_mblen(Symbol_value(S(utf_8)))(Symbol_value(S(utf_8)),sub,sub+ovector[ov_pos])));
+              pushSTACK(L_to_I(Encoding_mblen(Symbol_value(S(utf_8)))(Symbol_value(S(utf_8)),sub,sub+ovector[ov_pos+1])));
+              funcall(`PCRE::MAKE-MATCH-BOA`,2);
+              TheSvector(STACK_0)->data[pos] = value1;
+            }
+          VALUES1(popSTACK());
         }
-      VALUES1(popSTACK());
-    }
-  } else error_pcre(ret);
+      } else error_pcre(ret);
+    });
   skipSTACK(2);                 /* drop pattern & subject */
 }
 
@@ -457,4 +456,3 @@ void module__pcre__init_function_2 (module_t* module)
   pcre_malloc = malloc;
   pcre_free = free;
 }
-
