@@ -1,6 +1,6 @@
 ;; Module for regular expression searching/matching in CLISP
 ;; Bruno Haible 14.4.1995, 18.4.1995 -- 2003
-;; Sam Steingold 1999-10-28 -- 2007
+;; Sam Steingold 1999-10-28 -- 2009
 
 (defpackage "REGEXP"
   (:documentation
@@ -81,7 +81,7 @@
     (map nil (if extended
                (lambda (c)
                  (case c
-                   ((#\$ #\^ #\. #\* #\[ #\] #\\ #\+ #\?)
+                   ((#\$ #\^ #\. #\* #\[ #\] #\\ #\+ #\? #\( #\))
                     (vector-push-extend #\\ qstring)))
                  (vector-push-extend c qstring))
                (lambda (c)
@@ -151,4 +151,4 @@ The line is split with REGEXP-SPLIT using PATTERN."
 (defun regexp-matcher (pattern)
   "A valid value for *APROPOS-MATCHER* in the UTF-8 locale."
   (let ((compiled (regexp-compile pattern :extended t :ignore-case t)))
-    (lambda (name) (regexp-exec compiled name :boolean t))))
+    (lambda (name) (regexp-exec compiled name :return-type 'boolean))))
