@@ -1,5 +1,5 @@
-;;; Copyright (C) 2000-2008 by Sam Steingold
-;;; This file is a part of CLISP (http://clisp.cons.org), and, as such,
+;;; Sam Steingold 2000-2008, 2010-2011
+;;; This file is a part of CLISP (http://clisp.org), and, as such,
 ;;; is distributed under the GNU GPL (http://www.gnu.org/copyleft/gpl.html)
 
 (in-package "EXT")
@@ -17,8 +17,8 @@
     (:mozilla-window "mozilla" "-remote" "openURL(~a,new-window)")
     (:mozilla-tab "mozilla" "-remote" "openURL(~a,new-tab)")
     (:firefox "firefox" "~a")
-    (:firefox-window "firefox" "-remote" "openURL(~a,new-window)")
-    (:firefox-tab "firefox" "-remote" "openURL(~a,new-tab)")
+    (:firefox-window "firefox" "-new-window" "~a")
+    (:firefox-tab "firefox" "-new-tab" "~a")
     (:konqueror "kfmclient" "openURL" "~a")
     (:lynx "lynx" "~a")
     #+unix (:lynx-xterm "xterm" "-e" "lynx" "~a")
@@ -30,7 +30,10 @@
     #+macos (:default "open" "~a")
     (:mmm "mmm" "-external" "~a")
     (:mosaic "xmosaic" "~a")
-    (:emacs-w3 "gnudoit" "-q" "(w3-fetch \"~a\")"))
+    (:chromium "chromium-browser" "~a")
+    (:conkeror "conkeror" "~a")
+    (:opera "opera" "~a")
+    (:emacs "emacsclient" "--eval" "(browse-url ~s)"))
   "Alist of browsers and commands that invoke them.
 `~a' will be replaced with the URL to view.")
 (defvar *browser* nil
@@ -399,6 +402,7 @@ set *HTTP-PROXY*, and return it; otherwise just return *HTTP-PROXY*."
 (setf (documentation (find-package "CHARSET") 'sys::impnotes) "charset")
 (setf (documentation (find-package "CLOS") 'sys::impnotes) "classes")
 (setf (documentation (find-package "EXT") 'sys::impnotes) "ext-pac")
+#+MT (setf (documentation (find-package "THREADS") 'sys::impnotes) "mt")
 
 (defun clhs (symbol &key (browser *browser*) (out *standard-output*))
   "Dump the CLHS doc for the symbol."
