@@ -1,7 +1,7 @@
 ;;; Condition System for CLISP
 ;;; David Gadbois <gadbois@cs.utexas.edu> 30.11.1993
 ;;; Bruno Haible 24.11.1993, 2.12.1993 -- 2005
-;;; Sam Steingold 1998-2005
+;;; Sam Steingold 1998-2005, 2007-2011
 
 (in-package "COMMON-LISP")
 ;;; exports:
@@ -35,6 +35,8 @@
 (in-package "EXT")
 (export
  '(muffle-cerrors appease-cerrors exit-on-error with-restarts os-error
+   #+(or :win32 :cygwin) os-error-win32 os-error-code
+   os-stream-error os-file-error
    abort-on-error set-global-handler without-global-handlers
    source-program-error source-program-error-form source-program-error-detail
    simple-condition-format-string simple-charset-type-error retry)
@@ -87,12 +89,7 @@
 ;; DEFINE-CONDITION, CLtL2 p. 898
 (defmacro define-condition (&whole whole-form
                             name parent-types slot-specs &rest options)
-  (unless (symbolp name)
-    (error-of-type 'source-program-error
-      :form whole-form
-      :detail name
-      (TEXT "~S: the name of a condition must be a symbol, not ~S")
-      'define-condition name))
+  (setq name (check-not-declaration name 'define-condition))
   (unless (and (listp parent-types) (every #'symbolp parent-types))
     (error-of-type 'source-program-error
       :form whole-form
@@ -204,9 +201,11 @@
             (TEXT "~S ~S: superfluous arguments ~S")
             caller-name datum arguments)))
       datum)
+    (null                       ; os-error and subclasses
+      (apply #'make-condition default-type more-initargs))
     (symbol
       (apply #'make-condition datum arguments))
-    ((or string function) ; only this case uses default-type and more-initargs
+    ((or string function)
       (apply #'make-condition default-type
              :format-control datum
              :format-arguments arguments
@@ -230,7 +229,7 @@
   (handler-case
       (apply #'try-coerce-to-condition datum arguments
              caller-name default-type more-initargs)
-    (TYPE-ERROR (error) (signal error))
+    (TYPE-ERROR (error) (error error))
     (ERROR (error)
       ;; ANSI CL wants a type error here.
       (error-of-type 'type-error
@@ -311,7 +310,22 @@
 ;       |-- simple-warning
 ;       |
 ;       |-- style-warning
-;
+;       |   |
+;       |   |-- simple-style-warning
+;       |   |
+;       |   |------------+
+;       |                |
+;       |-- clos-warning |
+;           |            |
+;           |------------+-- clos-style-warning
+;           |                |
+;           |                |-- gf-already-called-warning
+;           |                |
+;           |                |-- gf-replacing-method-warning
+;           |                |
+;           |                |-- class-obsolescence-warning
+;           |
+;           |-- clos-novice-warning
 
 ;; X3J13 writeup <CONDITION-SLOTS:HIDDEN> wants the slot names to be hidden,
 ;; (e.g. no slot named `package', `stream', `pathname'), hence we prepend $.
@@ -326,7 +340,7 @@
     (define-condition program-error (error) ())
     ; all the other errors must be detected by the runtime system
 
-      ; statically detectable errors of a program, source available
+      ; statically detectable errors of a program, source available [CLISP specific]
       (define-condition source-program-error (program-error)
         (;; The "outer-most" bad form, i.e. the list whose first element is the
          ;; macro or special-operator name.
@@ -334,7 +348,6 @@
          ;; The "inner-most" detail of the bad form, e.g., when a string is
          ;; given as a variable name, this is the string, not the whole form.
          ($detail :initarg :detail :reader source-program-error-detail)))
-      ; CLISP specific
 
     ; not statically detectable errors in program control
     (define-condition control-error (error) ())
@@ -376,21 +389,18 @@
       (($datum         :initarg :datum         :reader type-error-datum)
        ($expected-type :initarg :expected-type :reader type-error-expected-type)))
 
-      ; when some keyword does not belong to one of the allowed keywords
+      ; when some keyword does not belong to one of the allowed keywords [CLISP specific]
       ; ANSI CL 3.5.1.4., 3.5.1.5. want this to be a subclass of PROGRAM-ERROR.
       (define-condition keyword-error (program-error type-error) ())
-      ; CLISP specific
 
-      ; when some character does not belong to a given character set
+      ; when some character does not belong to a given character set [CLISP specific]
       (define-condition charset-type-error (type-error) ())
-      ; CLISP specific
 
-      ; when an argument list in APPLY is dotted
+      ; when an argument list in APPLY is dotted [CLISP specific]
       ; ANSI CL 3.5.1.2. want this to be a subclass of PROGRAM-ERROR.
       ; For the use of APPLY in the expansion of the FORMATTER macro, this
       ; must also be a subclass of TYPE-ERROR.
       (define-condition argument-list-dotted (program-error type-error) ())
-      ; CLISP specific
 
     ; errors during operation on packages
     (define-condition package-error (error)
@@ -417,16 +427,28 @@
     (define-condition file-error (error)
       (($pathname :initarg :pathname :reader file-error-pathname)))
 
-    ; general OS errors
-    (define-condition os-error (error) ())
-    ; CLISP specific
+    ; general OS errors [CLISP specific]
+    (define-condition os-error (error)
+      (($code :initarg :code :reader os-error-code))
+      (:report (lambda (condition stream)
+                 (let ((code (os-error-code condition)))
+                   (format stream "~S(~A): ~A" (type-of condition) code
+                           (sys::strerror code))))))
+
+      #+(or :win32 :cygwin)
+      (define-condition os-error-win32 (os-error) ()
+        (:report (lambda (condition stream)
+                   (let ((code (os-error-code condition)))
+                     (format stream "~S(~A): ~A" (type-of condition) code
+                             (sys::format-message code))))))
+      (define-condition os-stream-error (os-error stream-error) ())
+      (define-condition os-file-error (os-error file-error) ())
 
   ; "Virtual memory exhausted"
   (define-condition storage-condition (serious-condition) ())
 
-  ; "User break"
+  ; "User break" [CLISP specific]
   (define-condition interrupt-condition (serious-condition) ())
-  ; CLISP specific
 
 ; conditions for which user notification is appropriate
 (define-condition warning () ())
@@ -434,6 +456,22 @@
   ; conditions which are a matter of programming style (not serious)
   (define-condition style-warning (warning) ())
 
+  ; CLOS user notifications [CLISP specific]
+  (define-condition clos:clos-warning (warning) ())
+    (define-condition clos:clos-novice-warning (clos:clos-warning) ())
+
+    (define-condition clos:clos-style-warning (style-warning clos:clos-warning) ())
+
+      ; CLOS: generic function is being modified, but has already been called
+      (define-condition clos:gf-already-called-warning (clos:clos-style-warning) ())
+
+      ; CLOS: replacing method in a GF
+      (define-condition clos:gf-replacing-method-warning (clos:clos-style-warning) ())
+
+      ; CLOS: class obsolescence
+      (define-condition clos:class-obsolescence-warning (clos:clos-style-warning) ())
+
+
 ;; These shouldn't be separate types but we cannot adjoin slots without
 ;; defining subtypes.
 
@@ -444,25 +482,13 @@
                     :reader simple-condition-format-control)
    ($format-arguments :initarg :format-arguments :initform nil
                       :reader simple-condition-format-arguments))
-  #|
   (:report
     (lambda (condition stream)
       (let ((fstring (simple-condition-format-control condition)))
-        (when fstring
+       (if fstring
           (apply #'format stream fstring
-                 (simple-condition-format-arguments condition))))))
-  |#
-)
-;; We don't use the :report option here. Instead we define a print-object
-;; method which will be executed regardless of the condition type's CPL.
-(clos:defmethod print-object :around ((condition simple-condition) stream)
-  (if (or *print-escape* *print-readably*)
-    (clos:call-next-method)
-    (let ((fstring (simple-condition-format-control condition)))
-      (if fstring
-        (apply #'format stream fstring (simple-condition-format-arguments condition))
-        (clos:call-next-method))))
-  condition)
+                (simple-condition-format-arguments condition))
+         (clos:call-next-method))))))
 
 ;; conditions usually created by ERROR or CERROR
 (define-condition simple-error (simple-condition error) ())
@@ -473,6 +499,16 @@
 ;; conditions usually created by WARN
 (define-condition simple-warning (simple-condition warning) ())
 
+(define-condition simple-style-warning (simple-condition style-warning) ())
+
+;; CLOS warnings
+(define-condition clos::simple-clos-warning (simple-condition clos:clos-warning) ())
+(define-condition clos::simple-clos-novice-warning (simple-condition clos:clos-novice-warning) ())
+(define-condition clos::simple-clos-style-warning (simple-condition clos:clos-style-warning) ())
+(define-condition clos::simple-gf-already-called-warning (simple-condition clos:gf-already-called-warning) ())
+(define-condition clos::simple-gf-replacing-method-warning (simple-condition clos:gf-replacing-method-warning) ())
+(define-condition clos::simple-class-obsolescence-warning (simple-condition clos:class-obsolescence-warning) ())
+
 ;; All conditions created by the C runtime code are of type simple-condition.
 ;; Need the following types. Don't use them for discrimination.
 (define-condition simple-serious-condition (simple-condition serious-condition) ())
@@ -497,7 +533,6 @@
 (define-condition simple-end-of-file (simple-error end-of-file) ())
 (define-condition simple-reader-error (simple-error reader-error) ())
 (define-condition simple-file-error (simple-error file-error) ())
-(define-condition simple-os-error (simple-error os-error) ())
 (define-condition simple-storage-condition (simple-condition storage-condition) ())
 (define-condition simple-interrupt-condition (simple-condition interrupt-condition) ())
 
@@ -537,7 +572,6 @@
     (end-of-file              . simple-end-of-file)
     (reader-error             . simple-reader-error)
     (file-error               . simple-file-error)
-    (os-error                 . simple-os-error)
     (storage-condition        . simple-storage-condition)
     (interrupt-condition      . simple-interrupt-condition)
     (warning                  . simple-warning)))
@@ -592,9 +626,10 @@
 |#
 
 ;; HANDLER-BIND, CLtL2 p. 898
-;; Since we can build handler frames only in compiled code
-;; there is SYS::%HANDLER-BIND which is synonymous to HANDLER-BIND except
-;; that SYS::%HANDLER-BIND only occurs in compiled code.
+;; Since we can build handler frames only in compiled code there is
+;; SYS::%HANDLER-BIND which is analogous to HANDLER-BIND except that the former
+;; only occurs in compiled code and has the normal function syntax.
+;; cf. compiler.lisp:c-HANDLER-BIND
 (defmacro handler-bind (clauses &body body)
   (let* ((typespecs (mapcar #'first clauses))
          (handlers (nconc (mapcar #'rest clauses) (list body)))
@@ -1181,20 +1216,20 @@
   (cond ((= place-numvalues 1)
          (format *debug-io*
                  (if instead-p
-                   (concatenate 'string "~&"
-                     (TEXT "Use instead~@[ of ~S~]: "))
+                   (concatenate 'string "~&" (TEXT "Use instead~@[ of ~S~]")
+                                (prompt-finish))
                    (prompt-for-new-value-string))
                  place)
-         (list (read *debug-io*)))
-        ((do ((ii 1 (1+ ii)) res)
-             ((> ii place-numvalues) (nreverse res))
-           (fresh-line *debug-io*)
-           (format *debug-io*
-                   (if instead-p
-                     (TEXT "Use instead of ~S [value ~D of ~D]: ")
-                     (TEXT "New ~S [value ~D of ~D]: "))
-                   place ii place-numvalues)
-           (push (read *debug-io*) res)))))
+         (list (eval (read *debug-io*))))
+        (t (do ((ii 1 (1+ ii)) res)
+               ((> ii place-numvalues) (nreverse res))
+             (fresh-line *debug-io*)
+             (format *debug-io*
+                     (if instead-p
+                       (TEXT "Use instead of ~S [value ~D of ~D]~A")
+                       (TEXT "New ~S [value ~D of ~D]~A"))
+                     place ii place-numvalues (prompt-finish))
+             (push (eval (read *debug-io*)) res)))))
 
 ;; CHECK-TYPE, CLtL2 p. 889
 (defmacro check-type (place typespec &optional (string nil) &environment env)
@@ -1220,11 +1255,11 @@
       place-oldvalue)
     ;; Only one restart. Call it STORE-VALUE, not CONTINUE, so that it's not
     ;; chosen by "continue".
-    (STORE-VALUE
+    (STORE-VALUE (new-value)
       :report (lambda (stream)
                 (format stream (report-one-new-value-string) place))
       :interactive (lambda () (prompt-for-new-value place place-numvalues))
-      (new-value) (funcall place-setter new-value))))
+      (funcall place-setter new-value))))
 
 ;; ASSERT, CLtL2 p. 891
 (defmacro assert (test-form &optional (place-list nil) (datum nil) &rest args
@@ -1267,7 +1302,7 @@
         "~A" error-string)
       (apply #'error condition-datum+args)) ; use coerce-to-condition??
     ;; Only one restart: CONTINUE.
-    (CONTINUE
+    (CONTINUE (&rest new-values)
       :REPORT (lambda (stream)
                 (apply #'format stream
                        (if (= (length place-list) 1)
@@ -1278,7 +1313,7 @@
                      (mapcan #'(lambda (place place-numvalues)
                                  (prompt-for-new-value place place-numvalues))
                              place-list place-numvalues-list))
-      (&rest new-values) (apply places-setter new-values))))
+      (apply places-setter new-values))))
 (defun simple-assert-failed (error-string &rest condition-datum+args) ; ABI
   (restart-case
     ;; No need for explicit association, see APPLICABLE-RESTART-P.
@@ -1288,10 +1323,9 @@
       (apply #'error condition-datum+args)) ; use coerce-to-condition??
     ;; Only one restart: CONTINUE.
     ;; But mark it as not meaningful, because it leads to user frustration.
-    (CONTINUE
+    (CONTINUE ()
       :REPORT (lambda (stream) (format stream (report-no-new-value-string)))
-      MEANINGFULP nil
-      ())))
+      MEANINGFULP nil)))
 
 (defun correctable-error (options condition)
   (let ((restarts
@@ -1324,15 +1358,26 @@
 (defun check-value (place condition)
   (let ((restarts
          (nconc
-          (list (make-restart
-                 :name 'USE-VALUE
-                 :report
-                   (lambda (stream)
-                     (format stream (report-one-new-value-string-instead)
-                             place))
-                 :interactive (lambda () (prompt-for-new-value place 1 t))
-                 :invoke-function
-                   (lambda (val) (return-from check-value (values val nil)))))
+          (when (eq place 'SYSTEM::PATHNAME-ENCODING)
+            (setq place '*pathname-encoding*) ; make it look nicer
+            (list (make-restart ; for direntry_to_string
+                   :name 'CONTINUE
+                   :report (lambda (stream)
+                             (format stream
+                                     (TEXT "Discard this directory entry")))
+                   :invoke-function
+                     (lambda () (return-from check-value (values nil 0))))))
+          (unless (eq place '*pathname-encoding*)
+            (list (make-restart
+                   :name 'USE-VALUE
+                   :report
+                     (lambda (stream)
+                       (format stream (report-one-new-value-string-instead)
+                               place))
+                   :interactive (lambda () (prompt-for-new-value place 1 t))
+                   :invoke-function
+                     (lambda (val)
+                       (return-from check-value (values val nil))))))
           (when (and (consp place) (eq 'fdefinition (car place)))
             (list (make-restart ; for check_fdefinition() only!
                    :name 'RETRY
@@ -1377,21 +1422,26 @@
 ;; we cannot use *LOAD-TRUENAME* because when the error is in a nested LOAD,
 ;; we will get the truename of the inner-most file instead of the current one
 (defun eval-loaded-form (obj file)
-  (restart-case (eval-loaded-form-low obj)
-    (skip
-        :report (lambda (out)
-                  (format out (TEXT "skip "))
-                  (if (compiled-function-p obj)
-                      (write (sys::closure-name obj) :stream out
-                             :pretty nil :escape nil)
-                      (write obj :stream out :pretty nil :escape nil
-                             :level 2 :length 3)))
+  (flet ((report (word obj out)
+           (write-string word out)
+           (if (compiled-function-p obj)
+               (write (closure-name obj) :stream out
+                      :pretty nil :escape nil)
+               (write obj :stream out :pretty nil :escape nil
+                      :level 2 :length 3))))
+    (restart-case (eval-loaded-form-low obj)
+      (skip ()
+        :report (lambda (out) (report (TEXT "skip ") obj out))
         :interactive default-restart-interactive
-        () (return-from eval-loaded-form 'skip))
-    (stop
+        (return-from eval-loaded-form 'skip))
+      (retry ()
+        :report (lambda (out) (report (TEXT "retry ") obj out))
+        :interactive default-restart-interactive
+        (return-from eval-loaded-form 'retry))
+      (stop ()
         :report (lambda (out) (format out (TEXT "stop loading file ~A") file))
         :interactive default-restart-interactive
-        () (return-from eval-loaded-form 'stop))))
+        (return-from eval-loaded-form 'stop)))))
 
 ;;; 29.4.3. Exhaustive Case Analysis
 
@@ -1472,6 +1522,14 @@
                   (case-errorstring keyform keyclauselist)
                   (case-expected-type keyclauselist)
                   env))
+    ;; for use in macros: when a file with generated ffi forms is compiled
+    ;; (prime example: gtk.lisp), any error message can become cryptic because
+    ;; the sources are efemeral - unless the whole form is printed in the error
+    (defmacro mecase (whole-form keyform &rest keyclauselist)
+      (simply-error 'CASE keyform keyclauselist
+                    `(format nil (TEXT "In form ~S~%~A") ,whole-form
+                             ,(case-errorstring keyform keyclauselist))
+                    (case-expected-type keyclauselist)))
 ) )
 (defun etypecase-failed (value errorstring expected-type) ; ABI
   (error-of-type 'type-error
@@ -1488,11 +1546,11 @@
         place-oldvalue))
     ;; Only one restart. Call it STORE-VALUE, not CONTINUE, so that it's not
     ;; chosen by "continue".
-    (STORE-VALUE
+    (STORE-VALUE (new-value)
       :report (lambda (stream)
                 (format stream (report-one-new-value-string) place))
       :interactive (lambda () (prompt-for-new-value place place-numvalues))
-      (new-value) (funcall place-setter new-value))))
+      (funcall place-setter new-value))))
 
 ;;; 29.4.11. Debugging Utilities
 
@@ -1588,9 +1646,7 @@
 
 (defvar *break-on-warnings* nil)
 
-;; WARN, CLtL2 p. 912
-;; (WARN format-string {arg}*)
-(defun warn (format-string &rest args)
+(defun warn-of-type (type format-string &rest args)
   (if (not *use-clcs*)
     (progn
       (fresh-line *error-output*)
@@ -1600,12 +1656,16 @@
       (elastic-newline *error-output*)
       (when *break-on-warnings* (funcall *break-driver* t)))
     (block warn
-      (let ((condition (coerce-to-condition format-string args 'warn 'simple-warning)))
+      (let ((condition (coerce-to-condition format-string args 'warn type)))
         (unless (typep condition 'warning)
           (error-of-type 'type-error
             :datum condition :expected-type 'warning
             (TEXT "~S: This is more serious than a warning: ~A")
             'warn condition))
+        (when (boundp '*warning-count*) (incf *warning-count*))
+        (when (and (boundp '*style-warning-count*)
+                   (typep condition 'style-warning))
+          (incf *style-warning-count*))
         (with-restarts ((MUFFLE-WARNING () (return-from warn)))
           (with-condition-restarts condition (list (find-restart 'MUFFLE-WARNING))
             (signal condition)))
@@ -1618,17 +1678,35 @@
         (elastic-newline *error-output*)
         (when *break-on-warnings*
           (with-restarts
-              ((CONTINUE
-                :report (lambda (stream)
-                          (format stream (TEXT "Return from ~S loop") 'break))
-                () (return-from warn)))
+              ((CONTINUE :report
+                 (lambda (stream)
+                   (format stream (TEXT "Return from ~S loop") 'break))
+                 () (return-from warn)))
             (with-condition-restarts condition (list (find-restart 'CONTINUE))
               ;; We don't call  (invoke-debugger condition)  because that
-              ;; would tell the user about a "Continuable error". Actually,
-              ;; it is only a warning!
+              ;; would tell the user about a "Continuable error".
+              ;; Actually, it is only a warning!
               (funcall *break-driver* nil condition nil)))))))
   nil)
 
+;; for X3J13 Issue COMPILER-DIAGNOSTICS:USE-HANDLER
+(defun c-warning (type format-string &rest args)
+  (let ((*error-output* *c-error-output*))
+    (apply #'warn-of-type type (string-concat "~A" format-string)
+           (c-current-location) args)))
+
+(defun c-cerror (location detail format-string &rest args)
+  (let ((*error-output* *c-error-output*))
+    (apply #'cerror-of-type (TEXT "Ignore the error and proceed")
+           'simple-source-program-error
+           :form *form* :detail detail
+           (string-concat location format-string)
+           args)))
+
+;; WARN, CLtL2 p. 912
+;; (WARN format-string {arg}*)
+(defun warn (format-string &rest args)
+  (apply #'warn-of-type 'simple-warning format-string args))
 
 #|
 Todo:
@@ -1746,8 +1824,7 @@ HANDLER should be funcallable (symbol or function).
 If it returns, the next applicable error handler is invoked.
 When HANDLER is nil, remove the global handler for CONDITION.
 Returns the added or removed method(s)."
-  (let ((clos::*warn-if-gf-already-called* nil)
-        (clos::*gf-warn-on-replacing-method* nil))
+  (let ((clos::*enable-clos-warnings* nil))
     (cond (handler              ; install handler
            (clos::do-defmethod 'global-handler
              (lambda (backpointer)
@@ -1755,8 +1832,7 @@ Returns the added or removed method(s)."
                (list
                 (lambda (condition)
                   ;; avoid infinite recursion by disabling the handler
-                  (let ((clos::*gf-warn-on-replacing-method* nil)
-                        (clos::*warn-if-gf-already-called* nil)
+                  (let ((clos::*enable-clos-warnings* nil)
                         (old-handler (set-global-handler condition-name nil)))
                     (unwind-protect (funcall handler condition)
                       (when old-handler
@@ -1791,3 +1867,44 @@ Returns the added or removed method(s)."
     `(let ((,handlers (set-global-handler nil nil)))
        (unwind-protect (progn ,@body)
          (set-global-handler ,handlers nil)))))
+
+;;; <http://www.lisp.org/HyperSpec/Body/dec_type.html>:
+;;;   A symbol cannot be both the name of a type and the name of a
+;;;   declaration. Defining a symbol as the name of a class, structure,
+;;;   condition, or type, when the symbol has been declared as a
+;;;   declaration name, or vice versa, signals an error.
+(defun check-not-type (symbol caller)
+  (loop
+    (setq symbol (check-symbol symbol caller))
+    (check-package-lock caller (symbol-package symbol) symbol)
+    (when (handler-bind ((error #'(lambda (c)
+                                    (declare (ignore c))
+                                    (return-from check-not-type symbol))))
+            (type-expand symbol))
+      (with-restarts ((use-value (new-value)
+                        :report
+                         (lambda (stream)
+                           (format stream (report-one-new-value-string-instead)
+                                   symbol))
+                         :interactive
+                          (lambda () (prompt-for-new-value symbol 1 t))
+                         (setq symbol new-value)))
+        (error (TEXT "~S: ~S defines a type, cannot be declared a ~S")
+               caller symbol 'declaration)))))
+
+(defun check-not-declaration (symbol caller)
+  (loop
+    (setq symbol (check-symbol symbol caller))
+    (check-package-lock caller (symbol-package symbol) symbol)
+    (unless (memq symbol (cdr (top-level-declarations)))
+      (return-from check-not-declaration symbol))
+    (with-restarts ((use-value (new-value)
+                      :report
+                       (lambda (stream)
+                         (format stream (report-one-new-value-string-instead)
+                                 symbol))
+                       :interactive
+                        (lambda () (prompt-for-new-value symbol 1 t))
+                       (setq symbol new-value)))
+      (error (TEXT "~S: ~S names a ~S, cannot name a type")
+             caller symbol 'declaration))))
