@@ -1,6 +1,6 @@
 ;;; Sources for CLISP DEFSTRUCT macro
 ;;; Bruno Haible 1988-2005
-;;; Sam Steingold 1998-2006
+;;; Sam Steingold 1998-2010, 2017
 ;;; German comments translated into English: Stefan Kain 2003-01-14
 
 (in-package "SYSTEM")
@@ -362,13 +362,11 @@
                           keyflag keywords keyvars keyinits keysvars
                           allow-other-keys auxvars auxinits)
         (analyze-lambdalist arglist
-          #'(lambda (detail errorstring &rest arguments)
-              (error-of-type 'source-program-error
-                :form whole-form
-                :detail detail
-                (TEXT "~S ~S: In ~S argument list: ~A")
-                'defstruct name ':constructor
-                (apply #'format nil errorstring arguments))))
+          #'(lambda (lalist detail errorstring &rest arguments)
+              (declare (ignore lalist)) ; use WHOLE-FORM instead
+              (sys::lambda-list-error whole-form detail
+                (TEXT "~S ~S: In ~S argument list: ~?")
+                'defstruct name ':constructor errorstring arguments)))
       (let* ((argnames
                ; The list of all arguments that are already supplied with
                ; values through the parameter list.
@@ -619,12 +617,7 @@
       (setq name (first name-and-options))
       (setq options (rest name-and-options)))
     ;; otherwise, name and options are already correct.
-    (unless (symbolp name)
-      (error-of-type 'source-program-error
-        :form whole-form
-        :detail name
-        (TEXT "~S: invalid syntax for name and options: ~S")
-        'defstruct name-and-options))
+    (setq name (check-not-declaration name 'defstruct))
     ;; name is a symbol, options is the list of options.
     ;; processing the options:
     (dolist (option options)
@@ -1100,8 +1093,8 @@
                              ',boa-constructors
                              ',copier-option
                              ',predicate-option)))
-           ,(if (eq type-option 'T)
-              `(CLOS::DEFINE-STRUCTURE-CLASS ',name
+           ,@(if (eq type-option 'T)
+              `((CLOS::DEFINE-STRUCTURE-CLASS ',name
                  ,namesform
                  ',keyword-constructor
                  ',boa-constructors
@@ -1112,8 +1105,11 @@
                           'clos::make-load-form-<structure-effective-slot-definition>))
                  (LIST ,@(make-load-form-slot-list
                           directslotlist slotdefaultdirectslots slotdefaultvars
-                          'clos::make-load-form-<structure-direct-slot-definition>)))
-              `(CLOS::UNDEFINE-STRUCTURE-CLASS ',name))
+                          'clos::make-load-form-<structure-direct-slot-definition>))
+                 ',docstring))
+              `((CLOS::UNDEFINE-STRUCTURE-CLASS ',name)
+                ;; see documentation.lisp: we map STRUCTURE to TYPE
+                (sys::%set-documentation ',name 'TYPE ',docstring)))
            ,@constructor-forms)
          ,@(if (and named-option predicate-option)
              (ds-make-pred predicate-option type-option name slotlist size))
@@ -1123,8 +1119,6 @@
                                   directslotlist)
                ,@(ds-make-writers name names type-option conc-name-option
                                   directslotlist)))
-         ;; see documentation.lisp: we map STRUCTURE to TYPE
-         (sys::%set-documentation ',name 'TYPE ,docstring)
          ,@(when (eq type-option 'T)
              (list
                (if print-object-option
