@@ -1,5 +1,6 @@
-;; win32 CLISP installation
-;; (c) 2001-2002  Sam Steingold, released as part of CLISP under GNU GPL.
+;;; win32 CLISP installation
+;;; Copyright (C) 2001-2005  Sam Steingold,
+;;; released as a part of CLISP under GNU GPL.
 
 ;; load this file in the directory where your CLISP distribution is located
 ;;  - to set the Registry appropriately
@@ -9,19 +10,12 @@
 (use-package "LDAP")
 
 (defvar *clisp-home* (namestring (default-directory)))
-(defvar *clisp-runtime*
+(defvar *clisp-linkset* "base")
+(defvar *clisp-driver* (ext:string-concat *clisp-home* "clisp.exe"))
+(defun clisp-cmd ()
   (if (position #\Space *clisp-home*)
-      (concatenate 'string "\"" *clisp-home* "\\base\\lisp.exe\"")
-      (concatenate 'string *clisp-home* "lisp.exe")))
-(defvar *clisp-some-args*
-  (concatenate 'string " -B \"" (substitute #\/ #\\ *clisp-home*) "\" -M "))
-(defvar *clisp-some-cmd*
-  (concatenate 'string *clisp-runtime* *clisp-some-args*))
-(defvar *clisp-args*
-  (concatenate 'string *clisp-some-args* "\""
-               *clisp-home* "/base/lispinit.mem\""))
-(defvar *clisp-cmd*
-  (concatenate 'string *clisp-runtime* *clisp-args*))
+      (ext:string-concat "\"" *clisp-driver* "\" -K " *clisp-linkset*)
+      (ext:string-concat *clisp-driver* " -K " *clisp-linkset*)))
 
 (defvar *eflags*
   (make-array 4 :element-type '(unsigned-byte 8) :initial-contents '(0 0 0 0)))
@@ -45,7 +39,7 @@
       (setf (dir-key-value cc "") "Compile with CLISP")
       (with-dir-key-open (cmd cc "command" :direction :output)
         (setf (dir-key-value cmd "")
-              (concatenate 'string *clisp-cmd* " -c \"%1\""))))))
+              (ext:string-concat (clisp-cmd) " -c \"%1\""))))))
 
 (defun add-fas-file (dkey)
   (format t "associating CLISP with FAS files under ~s~%" (dir-key-path dkey))
@@ -60,12 +54,12 @@
         (setf (dir-key-value ex "") "Execute with CLISP")
         (with-dir-key-open (cmd ex "command" :direction :output)
           (setf (dir-key-value cmd "")
-                (concatenate 'string *clisp-cmd* " \"%1\""))))
+                (ext:string-concat (clisp-cmd) " \"%1\""))))
       (with-dir-key-open (lo sh "Load_into_CLISP" :direction :output)
         (setf (dir-key-value lo "") "Load into CLISP")
         (with-dir-key-open (cmd lo "command" :direction :output)
           (setf (dir-key-value cmd "")
-                (concatenate 'string *clisp-cmd* " -i \"%1\"")))))))
+                (ext:string-concat (clisp-cmd) " -i \"%1\"")))))))
 
 (defun add-mem-file (dkey)
   (format t "associating CLISP with MEM files under ~s~%" (dir-key-path dkey))
@@ -80,7 +74,25 @@
         (setf (dir-key-value ex "") "Run with CLISP")
         (with-dir-key-open (cmd ex "command" :direction :output)
           (setf (dir-key-value cmd "")
-                (concatenate 'string *clisp-some-cmd* " \"%1\"")))))))
+                (ext:string-concat (clisp-cmd) " -M \"%1\"")))))))
+
+(defun make-clisp-shortcut (target)
+  (format t "~&writing <~a>..." target) (force-output)
+  (posix:make-shortcut
+   target :path *clisp-driver*
+   :arguments (ext:string-concat " -K " *clisp-linkset*)
+   :working-directory (namestring (user-homedir-pathname))
+   :description "GNU CLISP - ANSI Common Lisp implementation"
+   :icon (ext:string-concat *clisp-home* "doc\\clisp.ico"))
+  (format t "done~%"))
+
+(defun maybe-make-clisp-shortcut (where dir)
+  (let ((lnk-file (ext:string-concat dir "\\GNU CLISP.lnk")))
+    (when (y-or-n-p "Create a shortcut to CLISP ~A <~A>?" where lnk-file)
+      (make-clisp-shortcut lnk-file))))
+
+(when (y-or-n-p "Use the `full' linking set?")
+  (setq *clisp-linkset* "full"))
 
 (when (y-or-n-p "Associate types~:{ <~a>,~} with CLISP?" *lisp-type-map*)
   (with-dir-key-open (c1 :win32 "HKEY_CLASSES_ROOT")
@@ -91,24 +103,16 @@
     (add-fas-file c1)
     (add-mem-file c1)))
 
-(let* ((desktop (dir-key-single-value :win32 "HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders" "Common Desktop"))
-       (bat-file (concatenate 'string *clisp-home* "clisp.bat"))
-       (lnk-file (concatenate 'string desktop "\\CLISP.lnk"))
-       (url-file (concatenate 'string desktop "\\CLISP home.url")))
-  (when (y-or-n-p "Create a shorcut to CLISP on your desktop <~a>?" lnk-file)
-    (format t "~&writing <~a>..." lnk-file) (force-output)
-    (posix:make-shortcut
-     lnk-file :path *clisp-runtime* :arguments *clisp-args*
-     :working-directory (namestring (user-homedir-pathname))
-     :description "CLISP - ANSI Common Lisp implementation"
-     :icon (concatenate 'string *clisp-home* "doc\\clisp.ico"))
-    (format t "done~%"))
+(let (desktop programs url-file)
+  (with-dir-key-open (key :win32 "HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders" :direction :input)
+    (setq desktop (dir-key-value key "Common Desktop")
+          programs (dir-key-value key "Common Programs")))
+  (maybe-make-clisp-shortcut "on your desktop" desktop)
+  (maybe-make-clisp-shortcut "in your Start menu" programs)
+  (setq url-file (ext:string-concat desktop "\\GNU CLISP home.url"))
   (when (y-or-n-p "Create CLISP URL file on your desktop <~a>?" url-file)
     (with-open-file (url (substitute #\/ #\\ url-file) :direction :output)
       (format t "~&writing <~a>..." url-file) (force-output)
-      (format url "[Internetshortcut]~%URL=http://clisp.cons.org
+      (format url "[Internetshortcut]~%URL=http://clisp.org
 IconFile=~adoc\\clisp.ico~%" *clisp-home*)
-      (format t "done~%")))
-  (with-open-file (bat bat-file :direction :output)
-    (format bat "@echo off~%~a %1 %2 %3 %4 %5 %6 %7 %8 %9~%" *clisp-cmd*))
-  (format t "Please copy~%  <~a>~%to a directory in your PATH~%" bat-file))
+      (format t "done~%"))))
