@@ -3,7 +3,7 @@
 ;; <http://www.lisp.org/HyperSpec/Body/sec_6-1.html>
 ;; <http://www.lisp.org/HyperSpec/Body/mac_loop.html>
 ;; Bruno Haible 1991-2004
-;; Sam Steingold 1999-2005
+;; Sam Steingold 1999-2011
 
 (in-package "COMMON-LISP")
 (export '(loop loop-finish))
@@ -105,13 +105,14 @@
          (simple-type-p (cdr type)))))
 
 (defvar *helpvars*) ;; vector of auxiliary variables for destructuring
+(defvar *nullvar*)
 
 ;; (helpvar n) returns the (n+1)-st auxiliary variable (n>=0).
 ;; At least n auxiliary variable must already have been used.
 (defun helpvar (n)
   ;; '*helpvars*' is extended if necessary.
   (when (= n (fill-pointer *helpvars*))
-    (vector-push-extend (gensym) *helpvars*))
+    (vector-push-extend (gensym "PATTERN-") *helpvars*))
   (aref *helpvars* n))
 
 ;; (destructure pattern form) returns a list of lists (variable_i form_i).
@@ -134,7 +135,7 @@
                               (destructure-tree (cdr pattern) `(CDR ,helpvar) helpvar-count)))))))
     (or (destructure-tree pattern form 0)
         ; no variables -> must nevertheless evaluate form!
-        (list (list (helpvar 0) form)))))
+        (list (list (or *nullvar* (setq *nullvar* (gensym "NULLVAR-"))) form)))))
 
 ;; (default-bindings vars declspecs).
 ;; vars = (var ...) is a list of variables without init forms.
@@ -210,12 +211,17 @@
               ,form))))
   form)
 
+(defun wrap-unwind-protect (protect-forms form)
+  (if protect-forms
+      `(unwind-protect ,form ,@protect-forms)
+      form))
+
 ;; Variable containing the last test result, called "it".
 (defvar *last-it*)
 ;; Flag whether this variable is used.
 (defvar *used-it*)
 
-;;; (revadd a b c d)) ==
+;;; (revadd a b c d) ==
 ;;; (SETF A (REVAPPEND B (REVAPPEND C (REVAPPEND D A))))
 (defmacro revadd (place &rest forms)
   (labels ((expand (rest)
@@ -228,6 +234,7 @@
         (block-name 'NIL) ; Name des umgebenden BLOCKs
         (already-within-main nil) ; im zweiten Teil von {variables}* {main}* ?
         (*helpvars* (make-array 1 :fill-pointer 0 :adjustable t))
+        (*nullvar* nil)
         (*last-it* nil)
         (var-list nil)          ; all variables seen so far
         (acculist-var nil) ; Akkumulationsvariable für collect, append etc.
@@ -254,6 +261,7 @@
              ((NCONC NCONCING APPEND APPENDING)
               (unless (eq (loop-keywordp (caddr rest)) 'INTO)
                 (return nil))))))
+        (protect-forms '())
         (results nil)) ; alist (value-form . (clause list))
     (labels
       ((next-kw () ; Schaut, ob als nächstes ein Keyword kommt.
@@ -267,7 +275,7 @@
                                     (acons accuvar
                                            (gensym (symbol-name accuvar))
                                            accuvar-tailvar-alist))))))
-               (incrementvar (gensym)))
+               (incrementvar (gensym "ADDLIST-")))
            (push accuvar accu-vars-nil)
            (push tailvar accu-vars-nil)
            `(LET ((,incrementvar
@@ -476,7 +484,7 @@
                  ((IF WHEN UNLESS)
                   (pop body-rest)
                   (let* ((condition (parse-form kw))
-                         (it-var (gensym))
+                         (it-var (gensym "IT-"))
                          used-it
                          (true-form
                            (let ((*last-it* it-var) (*used-it* nil))
@@ -545,7 +553,7 @@
            (when (li-endtest-forms initialization)
              (setq seen-endtest t))
            (dolist (var (li-vars initialization))
-             (when (memq var var-list)
+             (when (and (not (eq var *nullvar*)) (memq var var-list))
                (error-of-type 'source-program-error
                  :form *whole* :detail var
                  (TEXT "~S: duplicate iteration variable ~S") *whole* var))
@@ -581,7 +589,7 @@
             (progn (setq already-within-main t) (push clause main-code))
             (let ((kw (loop-keywordp (first body-rest))))
               (case kw
-                ((WHILE UNTIL ALWAYS NEVER THEREIS)
+                ((WHILE UNTIL ALWAYS NEVER THEREIS REPEAT)
                  (pop body-rest)
                  (setq already-within-main t)
                  (let ((form (parse-form kw)))
@@ -595,11 +603,26 @@
                              (new-result 'T (list kw form))
                              `(WHEN ,form (RETURN-FROM ,block-name 'NIL)))
                            (THEREIS
-                             (let ((dummy (gensym)))
+                             (let ((dummy (gensym "THEREIS-")))
                                (new-result 'NIL (list kw form))
                                `(BLOCK ,dummy
                                   (RETURN-FROM ,block-name
-                                    (OR ,form (RETURN-FROM ,dummy NIL)))))))
+                                    (OR ,form (RETURN-FROM ,dummy NIL))))))
+                           ((REPEAT)
+                            (let ((var (gensym "COUNT-")))
+                              (note-initialization
+                               (make-loop-init
+                                :specform 'LET
+                                :bindings `((,var ,form))
+                                :declspecs nil
+                                :everytime nil
+                                :requires-stepbefore seen-endtest
+                                :preamble (preamble :start)
+                                :depends-preceding nil))
+                              (note-initialization
+                               (make-endtest `(UNLESS (PLUSP ,var)
+                                                (LOOP-FINISH))))
+                              `(SETQ ,var (1- ,var)))))
                          main-code)))
                 ((INITIALLY)
                  (pop body-rest)
@@ -614,7 +637,7 @@
                            (setq form `(PROGN ,@(parse-nonempty-progn kw))))
                           form)
                        finally-code))
-                ((WITH FOR AS REPEAT)
+                ((WITH FOR AS)
                  (pop body-rest)
                  (when already-within-main
                    (warn (TEXT "~S: ~A clauses should occur before the loop's main body")
@@ -689,10 +712,10 @@
                                      (setq step-function-form (parse-form 'by))
                                      (unless (and (function-form-p step-function-form)
                                                   (function-name-p (second step-function-form)))
-                                       (setq step-function-var (gensym))))
+                                       (setq step-function-var (gensym "BY-"))))
                                    (let ((var (if (and pattern (symbolp pattern)
                                                        (eq preposition 'ON))
-                                                  pattern (gensym))))
+                                                  pattern (gensym "LIST-"))))
                                      (push `(,var ,start-form) bindings)
                                      (when step-function-var
                                        (push `(,step-function-var ,step-function-form)
@@ -743,20 +766,20 @@
                                 (ACROSS
                                  (pop body-rest)
                                  (let ((vector-form (parse-form preposition))
-                                       (vector-var (gensym))
-                                       (index-var (gensym)))
+                                       (vector-var (gensym "VECTOR-"))
+                                       (index-var (gensym "INDEX-")))
                                    (push `(,vector-var ,vector-form) bindings)
                                    (push `(,index-var 0) bindings)
                                    (note-initialization
-                                     (make-endtest `(WHEN (>= ,index-var (LENGTH ,vector-var)) (LOOP-FINISH))))
+                                    (make-endtest `(WHEN (>= ,index-var (LENGTH ,vector-var)) (LOOP-FINISH))))
                                    (note-initialization
-                                     (make-loop-init
-                                       :specform 'LET
-                                       :bindings (destructure pattern `(AREF ,vector-var ,index-var))
-                                       :declspecs new-declspecs
-                                       :everytime t
-                                       :preamble (preamble :start)
-                                       :requires-stepbefore seen-endtest))
+                                    (make-loop-init
+                                     :specform 'LET
+                                     :bindings (destructure pattern `(AREF ,vector-var ,index-var))
+                                     :declspecs new-declspecs
+                                     :everytime t
+                                     :preamble (preamble :start)
+                                     :requires-stepbefore seen-endtest))
                                    (push (list index-var `(1+ ,index-var)) stepafter)))
                                 (BEING
                                  (pop body-rest)
@@ -802,10 +825,10 @@
                                                               ((HASH-KEY HASH-KEYS) t) (t nil)))))
                                               (loop-syntax-error 'using))
                                             (setq other-pattern (second (pop body-rest))))
-                                          (let ((state-var (gensym))
-                                                (nextp-var (gensym))
-                                                (nextkey-var (gensym))
-                                                (nextvalue-var (gensym)))
+                                          (let ((state-var (gensym "WHTI-"))
+                                                (nextp-var (gensym "MORE?"))
+                                                (nextkey-var (gensym "HASH-KEY-"))
+                                                (nextvalue-var (gensym "HASH-VALUE-")))
                                             (multiple-value-bind (nextmain-var nextother-var)
                                                 (case preposition
                                                   ((HASH-KEY HASH-KEYS) (values nextkey-var nextvalue-var))
@@ -845,9 +868,9 @@
                                                        ((PRESENT-SYMBOL PRESENT-SYMBOLS) '(:internal :external))
                                                        ((INTERNAL-SYMBOL INTERNAL-SYMBOLS) '(:internal))
                                                        ((EXTERNAL-SYMBOL EXTERNAL-SYMBOLS) '(:external))))
-                                              (state-var (gensym))
-                                              (nextp-var (gensym))
-                                              (nextsym-var (gensym))
+                                              (state-var (gensym "WPI-"))
+                                              (nextp-var (gensym "MORE?"))
+                                              (nextsym-var (gensym "SYMBOL-"))
                                               (form
                                                (case (next-kw)
                                                  ((IN OF) (pop body-rest)
@@ -875,7 +898,7 @@
                                             :requires-stepbefore seen-endtest))))))))
                                 (t
                                  (unless (symbolp pattern) (loop-syntax-error kw))
-                                 (unless pattern (setq pattern (gensym)))
+                                 (unless pattern (setq pattern (gensym "FOR-NUM-")))
                                  ;; ANSI CL 6.1.2.1.1 implies that the
                                  ;; start/end/by clauses can come in any
                                  ;; order, but only one of each kind.
@@ -909,7 +932,7 @@
                                             (pop body-rest)
                                             (setq step-end-form (parse-form preposition))
                                             (unless (constantp step-end-form)
-                                              (let ((step-end-var (gensym)))
+                                              (let ((step-end-var (gensym "LIMIT-")))
                                                 (push `(,step-end-var ,step-end-form) bindings)
                                                 (setq step-end-form step-end-var))))
                                            ((and (not step-by-p)
@@ -918,7 +941,7 @@
                                             (pop body-rest)
                                             (setq step-by-form (parse-form 'by))
                                             (unless (constantp step-by-form)
-                                              (let ((step-by-var (gensym)))
+                                              (let ((step-by-var (gensym "BY-")))
                                                 (push `(,step-by-var ,step-by-form) bindings)
                                                 (setq step-by-form step-by-var))))
                                            (t (return)))
@@ -934,7 +957,14 @@
                                                  (TEXT "~S: questionable iteration direction after ~A")
                                                  'loop (symbol-name kw))
                                                'down)
-                                             'up)))
+                                             (if (or step-start-p step-end-p step-by-p)
+                                               'up
+                                               (error-of-type 'source-program-error
+                                                 :form *whole* :detail
+                                                 (list kw pattern preposition)
+                                                 (TEXT "~S: invalid keyword ~A after ~A")
+                                                 'loop (symbol-name preposition)
+                                                 (symbol-name kw))))))
                                      ;; Determine start, unless given:
                                      (unless step-start-p
                                        (when (eq step-direction 'down)
@@ -986,22 +1016,7 @@
                       (dolist (initialization (nreverse initializations))
                         (when (li-everytime initialization)
                           (setf (li-everytime initialization) stepafter-code))
-                        (note-initialization initialization))))
-                   ((REPEAT)
-                    (let ((form (parse-form kw))
-                          (var (gensym)))
-                      (note-initialization
-                        (make-loop-init
-                          :specform 'LET
-                          :bindings `((,var ,form))
-                          :declspecs nil
-                          :everytime nil
-                          :requires-stepbefore seen-endtest
-                          :preamble (preamble :start)
-                          :depends-preceding t))
-                      (push `(SETQ ,var (1- ,var)) stepafter-code)
-                      (note-initialization
-                       (make-endtest `(UNLESS (PLUSP ,var) (LOOP-FINISH))))))))
+                        (note-initialization initialization))))))
                 (t (error-of-type 'source-program-error
                      :form *whole* :detail *whole*
                      (TEXT "~S: illegal syntax near ~S in ~S")
@@ -1058,11 +1073,11 @@
                   (if (li-later-depend initialization)
                     (progn ; double code: preamble and stepafter-code
                       (revadd (cdr (li-preamble initialization))
-                              endtest-forms initforms)
+                              initforms endtest-forms)
                       (revadd (cdr everytime) endtest-forms initforms))
                     (revadd stepbefore-code endtest-forms initforms))
                   (revadd (cdr (li-preamble initialization))
-                          endtest-forms initforms)))
+                          initforms endtest-forms)))
               ; Initialisierungsklausel nach initializations1 schaffen:
               (progn
                 (push
@@ -1134,6 +1149,7 @@
         `(MACROLET ((LOOP-FINISH () (LOOP-FINISH-ERROR)))
            (BLOCK ,block-name
              ,(wrap-initializations initializations1
+               (wrap-unwind-protect protect-forms
                 `(MACROLET ((LOOP-FINISH () '(GO END-LOOP)))
                    (TAGBODY
                      ,@(if preamble (nreverse (mapcar #'cdr preamble)))
@@ -1148,7 +1164,7 @@
                                accu-vars-nreverse)
                      (MACROLET ((LOOP-FINISH () (LOOP-FINISH-WARN)
                                   '(GO END-LOOP)))
-                       ,@(nreverse finally-code)))))))))))
+                       ,@(nreverse finally-code))))))))))))
 
 ;; Der eigentliche Macro:
 (defmacro loop (&whole whole &body body)
@@ -1156,7 +1172,7 @@
     ;; "extended" loop form
     (expand-loop whole body)
     ;; "simple" loop form
-    (let ((tag (gensym)))
+    (let ((tag (gensym "LOOP-")))
       `(BLOCK NIL (TAGBODY ,tag ,@body (GO ,tag))))))
 (defmacro loop-finish (&whole whole)
   (error (TEXT "~S is possible only from within ~S")
@@ -1170,7 +1186,7 @@
 
 ;; Run-Time-Support:
 
-(defun max-if (x y)
+(defun max-if (x y) ; ABI
   (if y (max x y) x))
-(defun min-if (x y)
+(defun min-if (x y) ; ABI
   (if y (min x y) x))
