@@ -12,7 +12,7 @@
     #endif
 
 
-    #if defined(ASM_UNDERSCORE) && !(defined(__CYGWIN32__) || defined(__MINGW32__))
+    #if defined(ASM_UNDERSCORE) && !(defined(__CYGWIN__) || defined(__MINGW32__))
 
       #define ALIGN  .align 3
     #else
@@ -32,4 +32,8 @@ FUNBEGIN(getSP)
         ret
 FUNEND()
 
+#if defined __linux__ || defined __FreeBSD__ || defined __FreeBSD_kernel__ || defined __DragonFly__
+        .section .note.GNU-stack,"",@progbits
+#endif
+
 
