@@ -2,6 +2,16 @@
 
 ;;; --- helpers for misc tests ---
 (defun princ-error (c) (format t "~&[~A]: ~A~%" (type-of c) c))
+(defmacro handler-return (block value)
+  `(lambda (c) (princ-error c) (return-from ,block ,value)))
+(defmacro check-os-error (form acceptable)
+  (let ((errvar (gensym "ERROR-")))
+    `(handler-case ,form
+       (os-error (,errvar)
+         (princ-error ,errvar)
+         (case (os-error-code ,errvar)
+           (,acceptable T)
+           (t (os-error-code ,errvar)))))))
 (defun show (object &key ((:pretty *print-pretty*) *print-pretty*))
   "Print the object on its own line and return it. Used in many tests!"
   (fresh-line) (prin1 object) (terpri) object)
@@ -21,12 +31,16 @@
   (ensure-directories-exist name :verbose t)
   (kill-down name))
 (defun cmd-args ()
-  "command and arg list for cloning this clisp process"
+  "Command and arg list for cloning this clisp process.
+This will not work right after (cd) if -M was a relative pathname."
   (let* ((argv (ext:argv))
-         (lispinit (aref argv (1+ (position "-M" argv :test #'string=)))))
+         (lispinit-pos (position "-M" argv :test #'string=))
+         (lispinit (and lispinit-pos (aref argv (1+ lispinit-pos))))
+         (args (list "-q" "-norc" "-B" (namestring *lib-directory*))))
     (values (aref argv 0)
-            (list "-q" "-norc" "-B" (namestring *lib-directory*) "-M"
-                  (namestring (merge-pathnames lispinit *lib-directory*))))))
+            (if lispinit
+                (nconc args (list "-M" lispinit))
+                args))))
 (export '(kill-down rmrf prepare-directory cmd-args))
 )
 (defun show-file (file)         ; return line count
@@ -40,11 +54,27 @@
   (delete-file (compile-file-pathname file))
   #+clisp (delete-file (make-pathname :type "lib" :defaults file)))
 (defun symbol-cleanup (s)
-  (setf (find-class s) nil) (makunbound s) (fmakunbound s) (unintern s))
+  #+CLISP (proclaim `(ext:notspecial ,s)) ; constants cannot be makunbound
+  #+CLISP (let ((tldl (sys::top-level-declarations)))
+            (when tldl
+              (setf (cdr tldl) (delete s (cdr tldl) :test #'eq))))
+  #+XCL(subr 84 ;sys::%p-set-cdr-content
+              s 0 (sys::%p-get-content 'sys::%void-value 0) 0)
+  #+ALLEGRO (setf (excl::symbol-bit s 'excl::.globally-special.) nil)
+  #+CMU (setf (ext:info variable kind s) ':global)
+  #+SBCL (setf (sb-int:info :variable :kind s) ':global)
+  #+OpenMCL (proclaim `(ccl::notspecial ,s))
+  (setf (find-class s) nil
+        (symbol-plist s) '())
+  (makunbound s)
+  (fmakunbound s)
+  (unintern s))
+(defun symbols-cleanup (l)      ; drop many symbols
+  (loop for s in l unless (symbol-cleanup s) collect s))
 (defvar *run-test-truename*)
 ;; export symbols used by tests
 (export '(princ-error show show-file finish-file post-compile-file-cleanup
-          type-error-handler
+          type-error-handler handler-return check-os-error symbols-cleanup
           symbol-cleanup *run-test-truename* with-ignored-errors diff-seq))
 ;;; end helpers
 
@@ -107,9 +137,11 @@
 (defvar *eval-method* :eval)
 (defvar *eval-out* nil)
 (defvar *eval-err* nil)
+(defvar *eval-times* nil)
 (defun my-eval (form)
   (when *eval-out* (get-output-stream-string *eval-out*))
   (when *eval-err* (get-output-stream-string *eval-err*))
+  (when *eval-times* (setq form `(times ,form)))
   (ecase *eval-method*
     (:eval (eval form))
     (:compile (funcall (compile nil `(lambda () ,form))))
@@ -276,6 +308,7 @@ NIL: sacla-style: forms should evaluate to non-NIL.")
                       (logname testname)
                       ((:own-package *run-test-own-package*)
                        *run-test-own-package*)
+                      ((:times *eval-times*) *eval-times*)
                  &aux (logfile (merge-extension *run-test-erg* logname))
                       error-count total-count *run-test-truename*)
   (with-open-file (s (merge-extension *run-test-type* testname)
@@ -310,26 +343,38 @@ NIL: sacla-style: forms should evaluate to non-NIL.")
       (format t "~s: see ~a~%" 'run-test logfile))
   (list logname total-count error-count))
 
-(defun report-results (res)
+(defun report-results (res &key (here (truename "./")))
   "res = list of RUN-TEST return values (testname total-count error-count)"
   (let ((error-count (reduce #'+ res :key #'third)))
     (format
      t "~&finished ~3d file~:p:~31T ~3:d error~:p out of~50T ~6:d test~:p~%"
      (length res) error-count (reduce #'+ res :key #'second))
-    (loop :with here = (truename "./") :for rec :in res :for count :upfrom 1 :do
+    (loop :for rec :in res :for count :upfrom 1 :do
       (format t "~&~3d ~25@a:~31T ~3:d error~:p out of~50T ~6:d test~:p~%"
               count (enough-namestring (first rec) here)
               (third rec) (second rec)))
     error-count))
 
-(defun run-some-tests (&key (dirlist '("./"))
-                       ((:eval-method *eval-method*) *eval-method*))
+;; see makemake.in:
+;; (run-some-tests :dirlist '("zlib" "pcre") :srcdir "../modules/")
+(defun run-some-tests (&key (dirlist '("./")) (srcdir "./") (outdir "./")
+                       ((:eval-method *eval-method*) *eval-method*)
+                       &aux #+clisp (custom:*merge-pathnames-ansi* t))
+  "Run tst files in DIRLIST under SRCDIR, writing erg under OUTDIR."
   (let ((files (mapcan (lambda (dir)
-                         (directory (make-pathname :name :wild
-                                                   :type *run-test-type*
-                                                   :defaults dir)))
-                       dirlist)))
-    (if files (report-results (mapcar #'run-test files))
+                         (directory (make-pathname
+                                     :name :wild :type *run-test-type*
+                                     :defaults (merge-pathnames dir srcdir))))
+                       dirlist))
+        (src-true (truename srcdir)))
+    (if files
+        (report-results
+         (mapcar (lambda (file)
+                   (run-test file :logname
+                             (merge-pathnames (enough-namestring file src-true)
+                                              outdir)))
+                 files)
+         :here outdir)
         (warn "no ~S files in directories ~S" *run-test-type* dirlist))))
 
 (defparameter *all-tests*
@@ -347,7 +392,7 @@ NIL: sacla-style: forms should evaluate to non-NIL.")
                       ("floeps")
     #-OpenMCL         ("format")
     #+CLISP           ("genstream")
-    #+XCL             ("hash")
+    #+(or CLISP XCL)  ("hash")
                       ("hashlong")
     #+CLISP           ("hashtable")
                       ("iofkts")
@@ -367,7 +412,7 @@ NIL: sacla-style: forms should evaluate to non-NIL.")
     #+CLISP           ("number2")
     #-(or AKCL ALLEGRO CMU OpenMCL) ("pack11")
     #+(or XCL CLISP)  ("path")
-    #+XCL             ("readtable")
+                      ("readtable")
     #-CMU             ("setf")
     #+(and CLISP SOCKETS) ("socket")
                       ("steele7")
@@ -378,7 +423,7 @@ NIL: sacla-style: forms should evaluate to non-NIL.")
                       ("symbols")
                       ("time")
     #+XCL             ("tprint")
-    #+XCL             ("tread")
+                      ("tread")
                       ("type")
     #-(or)            ("unportable")
     #+(and CLISP mt)  ("mt")
